#include "Initialization_Handler.H"

#include "IO_Handler.H"
#include "Model_Handler.H"
#include "Structure_Function.H"
#include "Intact.H"
#include "PDF_Handler.H"
#include "PDF_Base.H"

#include "Initial_State_Shower.H"

#include "Data_Read.H"
#include "Message.H"
#include "Scaling.H"

using namespace SHERPA;
using namespace MODEL;
using namespace BEAM;
using namespace PDF;
using namespace ATOOLS;
using namespace std;

struct Char_Array40 {
 char  s[40];
} ;

extern "C" {
  Char_Array40 visaje_();
}

Initialization_Handler::Initialization_Handler(string _path,string _file) : 
  m_path(_path), m_file(_file),
  p_model(NULL), p_beamspectra(NULL), p_isrhandler(NULL),
  p_harddecays(NULL), p_showerhandler(NULL), p_beamremnants(NULL), 
  p_fragmentation(NULL), p_hadrondecays(NULL), p_mihandler(NULL),
  p_iohandler(NULL)
{
  m_scan_istep=-1;  

  p_dataread         = new Data_Read(m_path+m_file);
  m_modeldat         = p_dataread->GetValue<string>("MODEL_DATA_FILE",string("Model.dat"));
  m_beamdat          = p_dataread->GetValue<string>("BEAM_DATA_FILE",string("Beam.dat"));
  m_isrdat           = p_dataread->GetValue<string>("ISR_DATA_FILE",string("ISR.dat"));
  m_medat            = p_dataread->GetValue<string>("ME_DATA_FILE",string("ME.dat"));
  m_midat            = p_dataread->GetValue<string>("MI_DATA_FILE",string("MI.dat"));
  m_decaydat         = p_dataread->GetValue<string>("DECAY_DATA_FILE",string("Decays.dat"));
  m_showerdat        = p_dataread->GetValue<string>("SHOWER_DATA_FILE",string("Shower.dat"));
  m_beamremnantdat   = p_dataread->GetValue<string>("BEAMREMNANT_DATA_FILE",string("Beam.dat"));
  m_fragmentationdat = p_dataread->GetValue<string>("FRAGMENTATION_DATA_FILE",string("Fragmentation.dat"));
  m_hadrondecaysdat  = p_dataread->GetValue<string>("FRAGMENTATION_DATA_FILE",string("Fragmentation.dat"));
  m_analysisdat      = p_dataread->GetValue<string>("ANALYSIS_DATA_FILE",string("Analysis.dat"));
}


Initialization_Handler::Initialization_Handler(int argc,char * argv[]) : 
  p_model(NULL), p_beamspectra(NULL), p_isrhandler(NULL),
  p_harddecays(NULL), p_showerhandler(NULL), p_beamremnants(NULL), 
  p_fragmentation(NULL), p_hadrondecays(NULL), p_mihandler(NULL),
  p_iohandler(NULL)
{
  m_path=std::string("./");
  m_file=std::string("Run.dat");

  m_scan_istep=-1;

  ExtractCommandLineParameters(argc, argv);

  if (m_mode>9000) {
    p_dataread         = new Data_Read(m_path+m_file);
    m_analysisdat      = p_dataread->GetValue<string>("ANALYSIS_DATA_FILE",string("Analysis.dat"));
    return;
  }
  p_dataread         = new Data_Read(m_path+m_file);
  m_modeldat         = p_dataread->GetValue<string>("MODEL_DATA_FILE",string("Model.dat"));
  m_beamdat          = p_dataread->GetValue<string>("BEAM_DATA_FILE",string("Beam.dat"));
  m_isrdat           = p_dataread->GetValue<string>("ISR_DATA_FILE",string("ISR.dat"));
  m_medat            = p_dataread->GetValue<string>("ME_DATA_FILE",string("ME.dat"));
  m_midat            = p_dataread->GetValue<string>("MI_DATA_FILE",string("MI.dat"));
  m_decaydat         = p_dataread->GetValue<string>("DECAY_DATA_FILE",string("Decays.dat"));
  m_showerdat        = p_dataread->GetValue<string>("SHOWER_DATA_FILE",string("Shower.dat"));
  m_beamremnantdat   = p_dataread->GetValue<string>("BEAMREMNANT_DATA_FILE",string("Beam.dat"));
  m_fragmentationdat = p_dataread->GetValue<string>("FRAGMENTATION_DATA_FILE",string("Fragmentation.dat"));
  m_hadrondecaysdat  = p_dataread->GetValue<string>("FRAGMENTATION_DATA_FILE",string("Fragmentation.dat"));
  m_analysisdat      = p_dataread->GetValue<string>("ANALYSIS_DATA_FILE",string("Analysis.dat"));

  CheckFlagConsitency();

  rpa.Init(m_path,m_file);
}


Initialization_Handler::~Initialization_Handler()
{
  while (m_analyses.size()>0) {
    delete m_analyses.begin()->second;
    m_analyses.erase(m_analyses.begin());
  }
  if (p_iohandler)     { delete p_iohandler;     p_iohandler     = NULL; }
  if (p_hadrondecays)  { delete p_hadrondecays;  p_hadrondecays  = NULL; }
  if (p_fragmentation) { delete p_fragmentation; p_fragmentation = NULL; }
  if (p_beamremnants)  { delete p_beamremnants;  p_beamremnants  = NULL; }
  if (p_showerhandler) { delete p_showerhandler; p_showerhandler = NULL; }
  if (p_harddecays)    { delete p_harddecays;    p_harddecays    = NULL; }
  if (p_mihandler)     { delete p_mihandler;     p_mihandler     = NULL; }
  if (p_isrhandler)    { delete p_isrhandler;    p_isrhandler    = NULL; }
  if (p_beamspectra)   { delete p_beamspectra;   p_beamspectra   = NULL; }
  if (p_model)         { delete p_model;         p_model         = NULL; }
  while (m_mehandlers.size()>0) {
    delete m_mehandlers.begin()->second;
    m_mehandlers.erase(m_mehandlers.begin());
  }
  PHASIC::Phase_Space_Handler::DeleteInfo();
}


bool Initialization_Handler::InitializeTheFramework(int nr)
{
  if (nr<=0) {
    ATOOLS::ParticleInit(m_path); 
  }

  bool okay = InitializeTheIO();
  if (m_mode==9999) {
    msg.Info()<<"Initialization_Handler::InitializeTheFramework :"<<std::endl
	      <<"   SHERPA will read in the events, the full framework is not needed."<<std::endl;
    InitializeTheAnalyses();
    return true;
  }

  okay      = okay && InitializeTheModel();  

  //  set masses and widths from command line
  SetParameter(nr);
  UpdateParameters();
    
  okay = okay && InitializeTheBeams();
  okay = okay && InitializeThePDFs();

  ATOOLS::Integration_Info *info=PHASIC::Phase_Space_Handler::GetInfo();
  p_isrhandler->AssignKeys(info);

  PHASIC::Phase_Space_Handler::GetInfo();

  if (!CheckBeamISRConsistency()) return 0.;
  
  okay = okay && InitializeTheHardDecays();
  okay = okay && InitializeTheMatrixElements();
  //  only if events:
  if (rpa.gen.NumberOfEvents()>0) {
    okay = okay && InitializeTheShowers();
    okay = okay && InitializeTheFragmentation();
    okay = okay && InitializeTheUnderlyingEvents();
    okay = okay && InitializeTheHadronDecays();
    okay = okay && InitializeTheBeamRemnants();
    okay = okay && InitializeTheAnalyses();
  }
  return okay;
}

bool Initialization_Handler::CheckBeamISRConsistency()
{
  if (p_model->Name()==std::string("ADD")) {
    double ms = p_model->ScalarConstant("M_s");
    if (ms<rpa.gen.Ecms()) {
      msg.Out()<<"WARNING in Initialization_Handler::CheckBeamISRConsistency :"<<std::endl
	       <<"   You are using the ADD model beyond its valid range ! "<<endl;
    }
  }

  double smin=0;
  double smax=sqr(rpa.gen.Ecms());
  smin = Max(smin,p_beamspectra->SprimeMin());
  smax = Min(smax,p_beamspectra->SprimeMax());
  if (p_isrhandler->On()) {
    smin = Max(smin,p_isrhandler->SprimeMin());
    smax = Min(smax,p_isrhandler->SprimeMax());
  }
  if (p_beamspectra->On()) {
    p_beamspectra->SetSprimeMin(smin);
  }
  string name=p_model->Name();
  if (name==std::string("ADD")) {
    double mcut2 = sqr(p_model->ScalarConstant("M_cut"));
    // if ISR & beam -> apply mcut on ISR only
    // if beam only  -> apply mcut on Beam
    smax = Min(smax,mcut2);
    if (p_isrhandler->On()) {
      p_isrhandler->SetFixedSprimeMax(smax);
      p_isrhandler->SetFixedSprimeMin(smin);
    } 
    else if (p_beamspectra->On()) {
      p_beamspectra->SetSprimeMax(smax);
    }
  }

  if (!(p_beamspectra->CheckConsistency(m_bunch_particles))) {
    msg.Error()<<"Error in Initialization of the Sherpa framework : "<<endl
	       <<"    Detected a mismatch of flavours from beams to bunches : "<<endl
	       <<"    "<<p_beamspectra->GetBeam(0)<<" -> "<<p_isrhandler->Flav(0)<<" and "
	       <<p_beamspectra->GetBeam(1)<<" -> "<<p_isrhandler->Flav(1)<<endl;
    return 0;
  }

  return 1;
}

bool Initialization_Handler::InitializeTheIO()
{  
  int filesize;
  std::vector<std::string> infiles, outfiles;
  infiles.push_back(p_dataread->GetValue<string>("SHERPA_INPUT",string("")));
  infiles.push_back(p_dataread->GetValue<string>("HEPMC_INPUT",string("")));
  infiles.push_back(p_dataread->GetValue<string>("HEPEVT_INPUT",string("")));
  outfiles.push_back(p_dataread->GetValue<string>("SHERPA_OUTPUT",string("")));
  outfiles.push_back(p_dataread->GetValue<string>("HEPMC_OUTPUT",string("")));
  outfiles.push_back(p_dataread->GetValue<string>("HEPEVT_OUTPUT",string("")));
  filesize  = p_dataread->GetValue<int>("FILE_SIZE",1000);
  p_iohandler = new IO_Handler(outfiles,infiles);
  return true;
}

bool Initialization_Handler::InitializeTheModel()
{
  if (p_model) { delete p_model; p_model = NULL; }
  Data_Read     * dataread     = new Data_Read(m_path+m_modeldat);
  // - add commandline parameter - !!
  Model_Handler * modelhandler = new Model_Handler();
  p_model                      = modelhandler->GetModel(dataread,m_path,m_modeldat);

  if (!p_model->RunSpectrumGenerator()) {
    msg.Error()<<"Error in Model_Initialization::Model_Initialization."<<endl
	       <<"    RunSpectrumGenerator() delivered false. Abort()."<<endl;
    abort();
  }

  delete modelhandler;
  delete dataread;  
  return 1;
}


bool Initialization_Handler::InitializeTheBeams()
{
  if (p_beamspectra) { 
    delete p_beamspectra; p_beamspectra = NULL; 
  }
  Data_Read * dataread = new Data_Read(m_path+m_beamdat);
  // - add commandline parameter - !!
  p_beamspectra        = new Beam_Spectra_Handler(dataread);
  msg.Info()<<"Initialized the beams "<<p_beamspectra->Type()<<endl;
  delete dataread;  
  return 1;
}


bool Initialization_Handler::InitializeThePDFs()
{
  if (p_isrhandler)  { delete p_isrhandler; p_isrhandler = NULL; }
  Data_Read * dataread     = new Data_Read(m_path+m_isrdat);
  // - add commandline parameter - !!
  PDF_Handler * pdfhandler = new PDF_Handler();
  PDF_Base *  pdfbase;
  ISR_Base ** isrbases     = new ISR_Base*[2];
  double  m_bunch_splimits[2];

  for (int i=0;i<2;++i) {
    pdfbase = pdfhandler->GetPDFLib(dataread,m_bunch_particles[i],i);
    if (pdfbase==NULL) isrbases[i] = new Intact(m_bunch_particles[i]);     
    else isrbases[i] = new Structure_Function(pdfbase,m_bunch_particles[i]);
    ATOOLS::rpa.gen.SetBunch(m_bunch_particles[i],i);
  }
  m_bunch_splimits[0] = dataread->GetValue<double>("ISR_SMIN",0.);
  m_bunch_splimits[1] = dataread->GetValue<double>("ISR_SMAX",1.);
  double kplimits[2];
  kplimits[0] = dataread->GetValue<double>("ISR_KPMIN",m_bunch_splimits[0]);
  kplimits[1] = dataread->GetValue<double>("ISR_KPMAX",m_bunch_splimits[1]);
  p_isrhandler = new ISR_Handler(isrbases,m_bunch_splimits,kplimits);
  msg.Info()<<"Initialized the ISR : "<<p_isrhandler->Type()<<endl;

  delete pdfhandler;
  delete dataread;

  if (!(p_beamspectra->CheckConsistency(m_bunch_particles))) {
    msg.Error()<<"Error in Environment::InitializeThePDFs()"<<endl
	       <<"   Inconsistent ISR & Beam:"<<endl
	       <<"   Abort program."<<endl;
    abort();
  }
  return 1;
}

bool Initialization_Handler::InitializeTheHardDecays()
{
  if (p_harddecays)    { delete p_harddecays;    p_harddecays    = NULL; }
  p_harddecays = new Hard_Decay_Handler(m_path,m_decaydat,m_medat,p_model);
  if (p_harddecays->GetMEHandler()!=NULL) {
    msg.Info()<<"Initialized the Hard_Decay_Handler. Its ME_Handler is : "
		      <<p_harddecays->GetMEHandler()->Name()<<"/"
		      <<p_harddecays->GetMEHandler()<<std::endl;
    m_mehandlers.insert(std::make_pair(std::string("HardDecays"),p_harddecays->GetMEHandler()));
  }
  return 1;
}

bool Initialization_Handler::InitializeTheMatrixElements()
{
  Matrix_Element_Handler * me = NULL;
  if (p_harddecays) {
    me = new Matrix_Element_Handler(m_path,m_medat,p_model,p_beamspectra,p_isrhandler,
				    p_harddecays->GetMEHandler());
  }
  else {
    me = new Matrix_Element_Handler(m_path,m_medat,p_model,p_beamspectra,p_isrhandler,NULL);
  }
  m_mehandlers.insert(std::make_pair(std::string("SignalMEs"),me)); 
  msg.Info()<<"Initialized the Matrix_Element_Handler for the hard processes :"<<me->Name()<<endl;
  return 1;
}

Matrix_Element_Handler * Initialization_Handler::GetMatrixElementHandler(std::string _key) { 
  MEHandlerIter pos = m_mehandlers.find(_key);
  if (pos!=m_mehandlers.end()) return pos->second;
  msg.Error()<<"Error in Initialization_Handler::GetMatrixElementHandler("<<_key<<") :"
		     <<"   Key not found. Return Null pointer."<<endl;
  return NULL;
}


bool Initialization_Handler::InitializeTheUnderlyingEvents()
{
  p_mihandler = new MI_Handler(m_path,m_midat,p_model,p_beamspectra,p_isrhandler);
  if (p_mihandler->Type()!=0)
    msg.Info()<<"Initialized the Multiple_Interactions_Handler (MI_Handler)."<<endl;
  Matrix_Element_Handler *mehandler = p_mihandler->HardMEHandler();
  if (mehandler!=NULL) {
    m_mehandlers.insert(std::make_pair(std::string("MIMEs"),mehandler)); 
    msg.Info()<<"Added the Matrix_Element_Handler for the u.e. :"<<mehandler->Name()<<endl;
  }
  return true;
}

bool Initialization_Handler::InitializeTheShowers()
{
  if (p_showerhandler) { delete p_showerhandler; p_showerhandler = NULL; }
  int maxjets     = GetMatrixElementHandler(std::string("SignalMEs"))->MaxJets();
  p_showerhandler = new Shower_Handler(m_path,m_showerdat,p_model,p_isrhandler,maxjets);
  msg.Info()<<"Initialized the Shower_Handler."<<endl;
  return 1;
}


bool Initialization_Handler::InitializeTheBeamRemnants() 
{
  if (p_beamremnants)  { delete p_beamremnants;  p_beamremnants  = NULL; }
  double scale=-4.0;
  if (p_showerhandler!=NULL) {
    if (p_showerhandler->GetApacic()->IniShower()) {
      scale=p_showerhandler->GetApacic()->IniShower()->CutOff();
    }
  }
  p_beamremnants = new Beam_Remnant_Handler(m_path,m_beamremnantdat,
					    p_isrhandler,p_beamspectra,scale);
  msg.Info()<<"Initialized the Beam_Remnant_Handler."<<endl;
  return 1;
}

bool Initialization_Handler::InitializeTheFragmentation() 
{
  if (p_fragmentation) { delete p_fragmentation; p_fragmentation = NULL; }
  p_fragmentation = new Fragmentation_Handler(m_path,m_fragmentationdat);
  msg.Info()<<"Initialized the Fragmentation_Handler."<<endl;
  return 1;
}

bool Initialization_Handler::InitializeTheHadronDecays() 
{
  if (p_hadrondecays)  { delete p_hadrondecays;  p_hadrondecays  = NULL; }
  p_hadrondecays  = new Hadron_Decay_Handler(m_path,m_hadrondecaysdat,
  					     p_fragmentation->GetLundInterface());
  return 1;
}

bool Initialization_Handler::InitializeTheAnalyses()
{
  if (rpa.gen.Analysis()<=0) return 1;
  std::string prefix=p_dataread->GetValue<std::string>("ANALYSIS_OUTPUT");
  if (prefix==NotDefined<std::string>()) prefix="";

  ifstream * from = new ifstream((m_path+m_analysisdat).c_str());
  if (!from->good()) {
    msg.Error()<<"Error in Initialization_Handler::InitializeTheAnalyses()."<<std::endl
	       <<"   File : "<<(m_path+m_analysisdat).c_str()<<" not found ! Abort program execution."<<endl;
    abort();
  }
  std::string buffer, phase;
  Analysis_Handler * sa = NULL;
  size_t pos;
  bool add;
  for (;;) {
    if (from->eof()) break;
    getline(*from,buffer);
    buffer += std::string(" ");
    if (buffer[0] != '%' && buffer.length()>0) {
      if (buffer.find("ANALYSIS_PHASE =")!=std::string::npos) {
	pos    = buffer.find("ANALYSIS_PHASE =");
	buffer = buffer.substr(pos+16);
	while(buffer.length()>0) {
	  if (buffer[0]==' ') buffer = buffer.substr(1);
	  else {
	    pos = buffer.find(string(" "));
	    if (pos!=string::npos) phase = buffer.substr(0,pos);
	    else phase = buffer;
	    break;
	  }
	}
	add = false;
	if (phase!=std::string("")) {
	  sa  = new Analysis_Handler(from,phase,prefix);
	  if (sa->On()) add = true;
	  else delete sa;
	}
	if (add) { 
	  int i=0;
	  std::string name;
	  do {
	    name=phase+std::string("_")+ATOOLS::ToString(++i);
	  } while (m_analyses.find(name)!=m_analyses.end());
	  m_analyses.insert(std::make_pair(name,sa)); 
	  msg.Info()<<"Initialized Analysis_Handler "<<name<<std::endl;
	}
      }
    }
  }
  return 1;
}

bool Initialization_Handler::CalculateTheHardProcesses()
{
  if (m_mode>9000) {
    msg.Out()<<"SHERPA will read in the events."<<std::endl
	     <<"   No cross sections for hard processes to be calculated."<<std::endl;
    return true;
  }

  int scalechoice = 0;
  if (p_showerhandler) {
    if (p_showerhandler->ISROn()) scalechoice += 1;
    if (p_showerhandler->FSROn()) scalechoice += 2;
  }
  Matrix_Element_Handler * me = GetMatrixElementHandler(std::string("SignalMEs"));
  msg.Out()<<"=========================================================================="<<std::endl
	   <<"Start calculating the hard cross sections. This may take some time.       "<<std::endl;
  int ok = me->CalculateTotalXSecs(scalechoice);
  if (ok && m_scan_istep!=-1) {
    AMEGIC::Process_Base * procs= me->GetAmegic()->Processes();
    msg.Out()<<ParameterValue()<<" ";
    for (size_t i=0; i<procs->Size();++i) {
      double xstot = (*procs)[i]->TotalXS()*rpa.Picobarn();
      msg.Out()<<xstot<<" ";
    }
    for (size_t i=0; i<procs->Size();++i) {
      msg.Out()<<"###"<<(*procs)[i]->Name();
    }
    msg.Out()<<endl;
  }
  if (ok) 
    msg.Out()<<"Calculating the hard cross sections has been successful.                  "<<std::endl
	     <<"=========================================================================="<<std::endl;
  else
    msg.Out()<<"Calculating the hard cross sections failed. Check this carefully.         "<<std::endl
	     <<"=========================================================================="<<std::endl;
  return ok;
}

void Initialization_Handler::SetParameter(int nr) {
  if (nr<0) return;

  if (nr!=m_scan_istep) 
    msg.Out()<<"WARNING: internal and external scan counter do not coincide "<<nr<<" vs. "<<m_scan_istep<<endl;

  double value=m_scan_value=m_scan_begin+(m_scan_end-m_scan_begin)*double(m_scan_istep)/double(m_scan_nsteps);

  MyStrStream s;
  string sval;
  if (m_scan_variable==string("ECMS")) {
    s<<value/2.;
    s>>sval;
    msg.Out()<<" Setting Ecms/2 to : "<<sval<<endl;
    Data_Read::SetCommandLine("BEAM_ENERGY_1",sval);
    Data_Read::SetCommandLine("BEAM_ENERGY_2",sval);
  }
  else if (m_scan_variable.find("MASS(")!=string::npos || m_scan_variable.find("WIDTH(")!=string::npos ) {
    s<<value;
    s>>sval;
    m_options[m_scan_variable]=sval;
    // make sure UpdateParameters() is called
  }   
  else {
    msg.Out()<<" Unknown Variable "<< m_scan_variable<<" in scan modus "<<endl;
    msg.Out()<<"  setting "<<m_scan_variable<<" = "<<value<<endl;
    s<<value; 
    s>>sval;
    m_options[m_scan_variable]=sval;    
    //    exit(1);
  }
  ++m_scan_istep;
}

int Initialization_Handler::ExtractCommandLineParameters(int argc,char * argv[])
{
  map<std::string,int> special_options;
  special_options["-V"]=12;
  special_options["--version"]=12;
  special_options["-?"]=13;
  special_options["-h"]=13;
  special_options["--help"]=13;
  special_options["-scan"]=14;
  special_options["-xsout"]=15;
  special_options["-eventout"]=16;

  special_options["PATH"]=101;
  special_options["RUNDATA"]=102;
  special_options["ECMS"]=103;
  special_options["EVTDATA"]=9999;

  

  for (int i=1; i<argc;++i) {
    int mode   = 0;
    string par = string(argv[i]);
    string key,value;
    int equal  = par.find("=");
    if (equal!=-1) {
      mode=1;
      value = par.substr(equal+1);
      key   = par = par.substr(0,equal);
      if (key.find("MASS")!=string::npos || key.find("WIDTH")!=string::npos) mode=100;
    }


    if (special_options.find(par)!=special_options.end()) 
      mode = special_options[par];

    // variables in dat files
    if (equal!=-1 && mode==1) {
      // perhaps check varible name first

      Data_Read::SetCommandLine(key,value);
    }
    
    // special variables
    if (mode>=100) {
      MyStrStream s;
      switch (mode) {
       case 101:
	if (value[value.length()-1]!='/') value+=std::string("/");
	m_path=value;
	break;
      case 102:
	  m_file=value;
	break;
      case 103:
	s<<value;
	double ecms;
	s>>ecms;
 	s<<ecms/2.;
	s>>value;
	Data_Read::SetCommandLine("BEAM_ENERGY_1",value);
	Data_Read::SetCommandLine("BEAM_ENERGY_2",value);
	break;
      case 9999:
	  m_mode       = 9999;
	  m_evtfile    = value;
	  msg.Out()<<" Sherpa will read in events from : "<<value<<endl;
        break;
      case 100:
	m_options[key] = value;
      }
    }
    else {
      // other option
      switch (mode) {
      case 12:
	{
	  // should call a version roution
	  msg.Out()<<" Sherpa Version 1.0.4"<<endl;
	  msg.Out()<<"   employing: "<<endl;
	  msg.Out()<<"    * AMEGIC++ Version 2.0.4 "<<endl;
	  msg.Out()<<"    * APACIC++ Version 2.0.4 "<<endl;

	  string pyver("6.214");
	  msg.Out()<<"    * Pythia Version "<<pyver<<endl;
	  Char_Array40 cid=visaje_();
	  msg.Out()<<"    * IsaJet Version "<<cid.s<<endl;
	}
	exit(0);
      case 13:
	msg.Out()<<" Help: "<<endl;
	msg.Out()<<" Sherpa [options] [<variable>=<value>] "<<endl;
	msg.Out()<<endl;
	msg.Out()<<" Possible options: "<<endl;
	msg.Out()<<"  -V,--version   prints the Version number"<<endl;
	msg.Out()<<"  -?,--help      prints this help message"<<endl;
// 	msg.Out()<<"  -xsout <filename> "<<endl;
// 	msg.Out()<<"                 sets a file where calculated cross sections should be printed to"<<endl;
// 	msg.Out()<<"  -eventout <filename> "<<endl;
// 	msg.Out()<<"                 sets a file where events should be printed to"<<endl;	
	msg.Out()<<"  -scan <variable> <startvalue> <stopvalue> <number of steps>"<<endl;
	msg.Out()<<"                 performs a parameter scan"<<endl<<endl;
	msg.Out()<<"        <variable> ... in addition to all parameters in configuration files"<<endl
		 <<"                       PATH, RUNDATA, ECMS, MASS(<kfcode>) can be used"<<endl;
	exit(0);
      case 14: 
	// scan
	m_mode=14;
	Data_Read::SetCommandLine("EVENTS","0");
	if (i+4<argc) {
	  m_scan_variable=argv[++i];
	  MyStrStream s;
	  s<<argv[++i];
	  s>>m_scan_begin;
	  s<<argv[++i];
	  s>>m_scan_end;
	  s<<argv[++i];
	  s>>m_scan_nsteps;
	  m_scan_istep=0;
	  msg.Out()<<" scanning "<<m_scan_variable
	      <<" from "<<m_scan_begin<<" to "<<m_scan_end
	      <<" in "<<m_scan_nsteps<<" steps"<<endl;
	}
	else {
	  msg.Error()<<"ERROR:  missing scan parameter -scan"<<endl;
	  msg.Error()<<"       try Sherpa -? for more information "<<endl;
	  exit(1);
	}
	break;
      case 15:
	// xsout
	break;
      case 16:
	// eventout
	break;
      }
    }
  }
  return m_mode;
}


void Initialization_Handler::CheckFlagConsitency()
{
  Data_Read dr(m_path+m_medat);
  int  sudweight = dr.GetValue<int>("SUDAKOV_WEIGHT",0);

  // if SUDAKOV_WEIGHT=On
  if (sudweight>0) {
    //  Run.dat
    long nevt = p_dataread->GetValue<long>("EVENTS",0);
    if (nevt<=0) {
      Data_Read::SetCommandLine("EVENTS","1");
    }

    //  ME.dat 
    Data_Read::SetCommandLine("SCALE_SCHEME","65");
    Data_Read::SetCommandLine("KFACTOR_SCHEME","65");
    Data_Read::SetCommandLine("COUPLING_SCHEME","Running_alpha_S");

    //  Shower.dat
    Data_Read::SetCommandLine("FSR_SHOWER","1");
    Data_Read::SetCommandLine("FS_LOSEJETVETO","1");
  }


  // check if all MI.dat / Decays.dat
  

}

int Initialization_Handler::UpdateParameters() 
{
  for (Parameter_Iterator it = m_options.begin(); it!=m_options.end() ; ++it) {
    MyStrStream s;
    string key=it->first;
    string value=it->second;
    msg.Info()<<" "<<key<<" = "<<value<<endl;
    int a=key.find("(")+1;
    int b=key.find(")")-a;
    msg.Tracking()<<"Flavour "<<key.substr(a,b);
    s<<key.substr(a,b);
    int kfc;
    s>>kfc;
    Flavour fl((kf::code)kfc);
    msg.Tracking()<<" : "<<fl<<endl;
    if (key.find("MASS")!=string::npos) {
      double mass=fl.Mass();
      msg.Tracking()<<" old mass = "<<mass<<endl;
      s<<value;
      s>>mass;
      msg.Tracking()<<" new mass = "<<mass<<endl;
      fl.SetMass(mass);
    }
    if (key.find(string("WIDTH"))!=string::npos) {
      msg.Tracking()<<"key:"<<key<<endl;
      double width=fl.Width();
      msg.Tracking()<<" old width = "<<width<<endl;
      s<<value;
      s>>width;
      msg.Tracking()<<" new width = "<<width<<endl;
      fl.SetWidth(width);
    }
  }
  return 1;
}
